#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/**** sample: interval timer :begin ****/
#include <time.h>
#include <sys/time.h>
/**** sample: interval timer :end ****/

#define MAIN_C
#include "exitfail.h"

#define SERIAL_BAUDRATE B115200
#define BUF_SIZE 2048

static int serial_fd = -1;
static struct termios old_tio;

static int terminated = 0;

#define BASENAME(p) ((strrchr((p), '/') ? : ((p) - 1)) +1)
#define ARRAY_SIZE(a) (sizeof(a) /sizeof((a)[0]))

#define __unused __attribute__((unused))

/**** sample: RN4020 command: begin ****/
static char iotsmart_id[] = "112233445566"; /* id(112233445566 is dummy) */
static unsigned char cmd_e_iotsmart[] = "E,0,112233445566\n"; /* connect iot smart */
static unsigned char cmd_chw_notify1[] = "CHW,0013,0100\n"; /* set notify iot smart custom1 */
static unsigned char cmd_chw_notify2[] = "CHW,0016,0100\n"; /* set notify iot smart custom2 */
/**** sample: RN4020 command: end ****/

/**** sample: RN4020 response: begin ****/
static unsigned char res_aok[] = "AOK"; /* response: AOK */
static unsigned char res_connect[] = "Connected"; /* response: connect */
/**** sample: RN4020 response: end ****/

static void terminate_sig_handler(__unused int signum)
{
	terminated = 1;
}

static void set_sig_handler(int sig_list[], ssize_t num, __sighandler_t handler)
{
	struct sigaction sa;
	int i;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handler;

	for(i=0; i<num; i++)
		if(sigaction(sig_list[i], &sa, NULL) < 0)
			exitfail_errno("sigaction");
}

static void restore_serial(void)
{
	int ret;

	ret = tcsetattr(serial_fd, TCSANOW, &old_tio);
	if(ret < 0) exitfail_errno("tcsetattr");
}

static void setup_serial(int fd)
{
	struct termios tio;
	int ret;

	ret = tcgetattr(fd, &old_tio);
	if(ret) exitfail_errno("tcsgetattr");

	if(atexit(restore_serial)) exitfail_errno("atexit");

	memset(&tio, 0, sizeof(tio));
	tio.c_iflag = IGNBRK | IGNPAR;
	tio.c_cflag = CS8 | CLOCAL | CREAD;
	tio.c_cc[VTIME] = 0;
	tio.c_cc[VMIN] = 1;
	ret = cfsetspeed(&tio, SERIAL_BAUDRATE);
	if(ret < 0) exitfail_errno("cfsetspeed");

	ret = tcflush(fd, TCIFLUSH);
	if(ret < 0) exitfail_errno("tcflush");

	ret = tcsetattr(fd, TCSANOW, &tio);
	if(ret) exitfail_errno("tcsetattr");
}

int main(int argc, char *argv[])
{
	int terminate_sig_list[] = {
		SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM
	};

/**** sample: variable: begin ****/
	ssize_t ret;
       unsigned char buf[BUF_SIZE];
       unsigned char buf1[BUF_SIZE];
	int i;
	int flag_connect, flag_aok;
	int cnt_cmp;
	FILE *fp;
	ssize_t analen;
/**** sample: variable: end ****/

/**** sample: interval timer: begin ****/
	struct itimerval itimer;
/**** sample: interval timer: end ****/

	exitfail_init();

/**** sample: Usage: begin ****/
	if(argc < 3) {
		printf("Usage: %s <device> <id>\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}
/**** sample: Usage: end ****/

	serial_fd = open(argv[1], O_RDWR | O_NOCTTY);
	if(serial_fd < 0) exitfail_errno("open");

	set_sig_handler(terminate_sig_list, ARRAY_SIZE(terminate_sig_list),
			terminate_sig_handler);

	setup_serial(serial_fd);

/**** sample: ble id: begin ****/
	strncpy(iotsmart_id, argv[2], sizeof(iotsmart_id));
	for(i = 0; i < ((int)sizeof(iotsmart_id) - 1); i++) 
		cmd_e_iotsmart[4 + i] = iotsmart_id[i];
/**** sample: ble id: end ****/

/**** sample: interval timer: begin ****/
	itimer.it_value.tv_sec = itimer.it_interval.tv_sec = 10;
	itimer.it_value.tv_usec = itimer.it_interval.tv_usec = 0;
/**** sample: interval timer: end ****/

/**** sample: RN4020 command(E to connect IoT Smart): begin ****/
	ret = write(serial_fd, cmd_e_iotsmart, sizeof(cmd_e_iotsmart) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_e_iotsmart);

	flag_connect = 0;
	cnt_cmp = 0;
	analen = 0;

	fp = fopen("/root/log_ble_connect.txt", "w");
	fclose(fp);

	while(!terminated && flag_connect != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_connect != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."CoConnected" */
			if(buf[i] == res_connect[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				analen = 0;
			}
			if(cnt_cmp == sizeof(res_connect) - 1) {
				flag_connect = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_connect == 1) {
			printf("\n-------- Receive: %s\n", res_connect);
//			printf("%s\n", buf1);
		}
	}

	fp = fopen("/root/log_ble_connect.txt", "w");
	fprintf(fp, "id=%s", iotsmart_id);
	fclose(fp);	
/**** sample: RN4020 command(E to connect IoT Smart): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Custom1 Notify): begin ****/
	ret = write(serial_fd, cmd_chw_notify1, sizeof(cmd_chw_notify1) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_chw_notify1);

	flag_aok = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_aok != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_aok != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."AOAOK" */
			if(buf[i] == res_aok[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				analen = 0;
			}
			if(cnt_cmp == sizeof(res_aok) - 1) {
				flag_aok = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_aok == 1) {
			printf("\n-------- Receive: %s\n", res_aok);
//			printf("%s\n", buf1);
		}
	}
/**** sample: RN4020 command(CHW to set IoT Smart: Custom1 Notify): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Custom2 Notify): begin ****/
	ret = write(serial_fd, cmd_chw_notify2, sizeof(cmd_chw_notify2) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_chw_notify2);

	flag_aok = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_aok != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_aok != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."AOAOK" */
			if(buf[i] == res_aok[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				analen = 0;
			}
			if(cnt_cmp == sizeof(res_aok) - 1) {
				flag_aok = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_aok == 1) {
			printf("\n-------- Receive: %s\n", res_aok);
//			printf("%s\n", buf1);
		}
	}
/**** sample: RN4020 command(CHW to set IoT Smart: Custom2 Notify): end ****/
	printf("\n######## Connected: IoT Smart ########\n");

	return EXIT_SUCCESS;
}

